#ifndef FONT_H_
#define FONT_H_

#include <map>
#include <string>
#include <msp/parser/loader.h>

namespace Msp {
namespace GL {

class Texture2D;

class Font
{
public:
	class Loader: public Msp::Parser::Loader
	{
	public:
		Loader(Font &);
	private:
		Font &font;

		void texture(const std::string &);
		void glyph(unsigned);
	};

	Font();
	void  set_texture(const Texture2D &);
	void  add_glyph(wchar_t, float, float, float, float, float, float, float, float);
	float get_string_width(const std::string &) const;
	void  draw_glyph(wchar_t);
	void  draw_string(const std::string &) const;
	void  draw_multiline(const std::string &) const;
	~Font();
private:
	struct Glyph
	{
		class Loader: public Msp::Parser::Loader
		{
		public:
			Loader(Glyph &);
			Glyph &get_object() { return glyph; }
		private:
			Glyph &glyph;

			void texcoords(float, float, float, float);
			void size(float, float);
		};

		wchar_t code;
		float x1,y1;
		float x2,y2;
		float w,h;
		float descent;
		float advance;
	};
	typedef std::map<wchar_t, Glyph> GlyphMap;

	const Texture2D *tex;
	bool     own_tex;
	GlyphMap glyphs;

	void prepare(float *) const;
	void draw_glyph(wchar_t, float *, float &) const;
};

} // namespace GL
} // namespace Msp

#endif
